SystemOrganization addCategory: #'DuinoBot-Core'!
SystemOrganization addCategory: #'DuinoBot-Core-Firmware'!
SystemOrganization addCategory: #'DuinoBot-Core-Hid'!

ArduinoDirectMode subclass: #DuinoBotDirectMode
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DuinoBot-Core-Firmware'!

!DuinoBotDirectMode methodsFor: 'as yet unclassified' stamp: 'Richo 4/18/2012 17:00'!
controlFirmwareInstalled
	| time timeout |
	time := Time millisecondClockValue.
	timeout := 5000.
	[100 milliSeconds asDelay wait.
	('*HOLA, SOY DUINOBOT*' match: port readString)
		ifTrue: [
			port nextPutAll: (ByteArray with: 255).
			[port readString = ''] whileFalse.
			^ self]]
		doWhileTrue: [Time millisecondClockValue - time < timeout].
	^ FirmwareNotInstalled signal! !

!DuinoBotDirectMode methodsFor: 'as yet unclassified' stamp: 'Richo 8/31/2011 17:33'!
motorDC: motorIndex speed: pinValue
	| speed |
	speed := (1.275 * pinValue + 127.5) rounded.
	self send: {self asCommand: self class rqMotorDC.
						self asArgument: motorIndex.
						self asArgument: (speed bitAnd: 2r01111111).
						self asArgument: (speed bitAnd: 2r10000000) >> 7} asByteArray! !

Arduino subclass: #DuinoBot
	instanceVariableNames: 'motorPins'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DuinoBot-Core'!

!DuinoBot class methodsFor: 'as yet unclassified' stamp: 'Richo 5/15/2012 15:10'!
defaultProtocol
	^ DuinoBotDirectMode! !

!DuinoBot methodsFor: 'initialize-release' stamp: 'Richo 3/6/2012 09:57'!
initialize
	super initialize.
	motorPins := Dictionary newFrom: ((0 to: 1)
				collect: [:i | i -> (DuinoBotMotorPin on: self number: i)])! !

!DuinoBot methodsFor: 'motors' stamp: 'Richo 8/31/2011 15:20'!
motorPin: index
	^ motorPins at: index ifAbsent: [self inaccessiblePinError]! !

!DuinoBot methodsFor: 'accessing' stamp: 'Richo 10/24/2012 15:44'!
motorPins
	^ motorPins values
! !

!DuinoBot methodsFor: 'accessing' stamp: 'Richo 10/24/2012 15:56'!
pins
	^ super pins, self motorPins! !

!DuinoBot methodsFor: 'initialize-release' stamp: 'Richo 4/18/2012 13:34'!
reconfigurePins
	"Overriden to avoid DuinoBot issue on Linux"
	digitalPins values , analogPins values
		do: [:each | 
			100 milliSeconds asDelay wait.
			each refresh]! !

ArduinoType subclass: #DuinoBotType
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DuinoBot-Core'!

!DuinoBotType class methodsFor: 'accessing types' stamp: 'Richo 11/23/2012 12:34'!
default
	^ self at: #'DuinoBot.v1.x.HID'! !

!DuinoBotType class methodsFor: 'accessing types' stamp: 'Richo 11/23/2012 12:36'!
hidTypes
	^ {self at: #'DuinoBot.v1.x.HID'}! !

!DuinoBotType class methodsFor: 'class initialization' stamp: 'Richo 11/23/2012 13:38'!
initialize
	"
	self initialize.
	"
	types := Dictionary new.

	types
		at: #DuinoBotCDC
		put: (DuinoBotType
			friendlyNames: #('duinobot 1.x' )
			fCpu: 16000000
			mcu: 'atmega32u4'
			uploadRate: 115200
			firmataRate: 57600
			avrdudeProgrammer: 'avr109').

	types
		at: #'DuinoBot.v1.x.HID'
		put: (DuinoBotType
			friendlyNames: #('duinobot 1.x hid' )
			fCpu: 16000000
			mcu: 'atmega32u4'
			uploadRate: 115200
			firmataRate: 57600
			avrdudeProgrammer: 'avr109')! !

!DuinoBotType methodsFor: 'testing' stamp: 'Richo 11/23/2012 12:36'!
isHID
	^ self class hidTypes includes: self! !

Object subclass: #HID
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DuinoBot-Core-Hid'!

!HID class methodsFor: 'api' stamp: 'Richo 12/2/2012 17:35'!
apiClose: device
"void HID_API_EXPORT HID_API_CALL hid_close(hid_device *device);"
	<apicall: void 'hid_close' (long) module: 'hidapi.dll'>
	^ self externalCallFailed! !

!HID class methodsFor: 'api' stamp: 'Richo 12/2/2012 16:18'!
apiExit
	<apicall: long 'hid_exit' () module: 'hidapi.dll'>
	^ self externalCallFailed! !

!HID class methodsFor: 'api' stamp: 'Richo 12/2/2012 16:17'!
apiInit	
	<apicall: long 'hid_init' () module: 'hidapi.dll'>
	^ self externalCallFailed! !

!HID class methodsFor: 'api' stamp: 'Richo 12/2/2012 16:25'!
apiOpen: vendorId with: productId with: serialNumber
"HID_API_EXPORT hid_device * HID_API_CALL hid_open(unsigned short vendor_id, unsigned short product_id, const wchar_t *serial_number);"
	<apicall: long 'hid_open' (ushort ushort short*) module: 'hidapi.dll'>
	^ self externalCallFailed! !

!HID class methodsFor: 'api' stamp: 'Richo 12/2/2012 17:31'!
apiRead: device with: buffer with: bufferSize
"int  HID_API_EXPORT HID_API_CALL hid_read(hid_device *device, unsigned char *data, size_t length);"
	<apicall: long 'hid_read' (long char* ulong) module: 'hidapi.dll'>
	^ self externalCallFailed! !

!HID class methodsFor: 'api' stamp: 'Richo 12/2/2012 17:29'!
apiReadTimeout: device with: buffer with: bufferSize with: timeout
"int HID_API_EXPORT HID_API_CALL hid_read_timeout(hid_device *dev, unsigned char *data, size_t length, int milliseconds);"
	<apicall: long 'hid_read_timeout' (long char* ulong long) module: 'hidapi.dll'>
	^ self externalCallFailed! !

!HID class methodsFor: 'api' stamp: 'Richo 12/4/2012 14:16'!
apiWrite: device with: buffer with: bufferSize
"int  HID_API_EXPORT HID_API_CALL hid_write(hid_device *device, const unsigned char *data, size_t length);"
	<apicall: long 'hid_write' (long char* ulong) module: 'hidapi.dll'>
	^ self externalCallFailed! !

!HID class methodsFor: 'utils' stamp: 'Richo 12/4/2012 14:04'!
close: ptr
	self apiClose: ptr.
	self apiExit = -1
		ifTrue: [^ self error: 'HID finalization error' translated]
	! !

!HID class methodsFor: 'utils' stamp: 'Richo 12/4/2012 14:03'!
open: vid with: pid
	| ptr |
	self apiInit = -1
		ifTrue: [^ self error: 'HID initialization error' translated].
	ptr := self apiOpen: vid with: pid with: nil.
	ptr = 0
		ifTrue: [^ self error: 'HID opening device error' translated].
	^ ptr! !

!HID class methodsFor: 'utils' stamp: 'Richo 12/4/2012 14:21'!
read: ptr
	| bufSize buf bytesRead |
	bufSize := 8.	
	buf := ByteArray new: bufSize.
	bytesRead := self apiReadTimeout: ptr with: buf with: bufSize with: 0.
	^ bytesRead caseOf: {
		[bufSize] -> [buf].
		[0] -> [#() asByteArray].
		[-1] -> [self error: 'HID reading error' translated]
	} otherwise: [buf copyFrom: 1 to: bytesRead]! !

!HID class methodsFor: 'utils' stamp: 'Richo 12/4/2012 14:39'!
write: ptr with: data
	| bytesWritten |
	bytesWritten := self apiWrite: ptr with: data with: data size.
	bytesWritten = -1
		ifTrue: [^ self error: 'HID writing error' translated].
	^ bytesWritten! !

Object subclass: #HIDConnection
	instanceVariableNames: 'device'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DuinoBot-Core-Hid'!

!HIDConnection methodsFor: 'connection' stamp: 'Richo 12/4/2012 14:42'!
connectOnPort: portName
	('HID' match: portName) ifFalse: [^ self error: 'HID invalid port' translated].
	device := HID open: 16r3EB with: 16r204F.! !

!HIDConnection methodsFor: 'connection' stamp: 'Richo 12/4/2012 14:28'!
disconnect
	[HID close: device]
		ensure: [device := nil]! !

!HIDConnection methodsFor: 'connection' stamp: 'Richo 12/4/2012 14:36'!
receive
	| result |
	result := OrderedCollection new.
	8 timesRepeat: [| data |
		data := HID read: device.
		data first = 16rB1
			ifTrue: [result addAll: data allButFirst]].
	^ result asByteArray! !

!HIDConnection methodsFor: 'connection' stamp: 'Richo 12/4/2012 14:39'!
send: data
	self flag: #Richo. "Check that the entire data has been sent."
	HID send: device with: (#(16rB1), data) asByteArray! !

AttachableDevice subclass: #RGMotor
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DuinoBot-Core'!

!RGMotor methodsFor: 'as yet unclassified' stamp: 'Richo 9/7/2011 11:15'!
brake
	self controlAttaching; reconfigureIfNecessary.
	pins first value: 0! !

!RGMotor methodsFor: 'as yet unclassified' stamp: 'Richo 9/7/2011 11:15'!
configure
	pins first refresh! !

!RGMotor methodsFor: 'as yet unclassified' stamp: 'Richo 9/7/2011 11:15'!
deleteConfiguration
	pins first value: 0! !

!RGMotor methodsFor: 'accessing' stamp: 'Richo 10/24/2012 17:15'!
motorID
	^ pins first number! !

!RGMotor methodsFor: 'as yet unclassified' stamp: 'Richo 9/7/2011 11:10'!
speed
	self controlAttaching; reconfigureIfNecessary.
	^ pins first value! !

!RGMotor methodsFor: 'as yet unclassified' stamp: 'Richo 9/7/2011 11:11'!
speed: aNumber
	self controlAttaching; reconfigureIfNecessary.
	pins first value: aNumber! !

ArduinoPin subclass: #DuinoBotMotorPin
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DuinoBot-Core'!

!DuinoBotMotorPin methodsFor: 'as yet unclassified' stamp: 'Richo 3/6/2012 09:56'!
refresh
	self protocol motorDC: number speed: value
! !

!DuinoBotMotorPin methodsFor: 'as yet unclassified' stamp: 'Richo 9/1/2011 14:19'!
value
	^ value! !

!DuinoBotMotorPin methodsFor: 'as yet unclassified' stamp: 'Richo 8/31/2011 15:28'!
value: speed
	value := (speed min: 100 max: -100).
	self refresh! !
DuinoBotType initialize!
